from __future__ import annotations


def get_category_colors() -> dict[str, str]:
    return {
        'World': '#0077BB',
        'STEM': '#AA4499',
        'Sports': '#228833',
        'Economics': '#EE7733',
        'Politics': '#CC3311',
        'Financials': '#009988',
        'Crypto': '#33BBEE',
        'Culture': '#EE3377',
        'Unknown': '#BBBBBB',
    }
